"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
const https_1 = __importDefault(require("https"));
const http_1 = __importDefault(require("http"));
const ws_1 = __importDefault(require("ws"));
const events_1 = require("events");
const telnet_1 = __importDefault(require("./telnet"));
const uuid_1 = require("uuid");
class WST extends events_1.EventEmitter {
    constructor(socket) {
        super();
        this.UUID = uuid_1.v4();
        this.options = new telnet_1.default.TelnetOptionMatrix();
        this.responder = null;
        this.socket = socket;
        this.socket.on("open", () => this.emit("open"));
        this.socket.on("close", (code, reason) => this.emit("close", code, reason));
        this.socket.on("error", (err) => this.emit("error", err));
        this.socket.on("ping", (data) => this.emit("ping", data));
        this.socket.on("pong", (data) => this.emit("pong", data));
        this.socket.on("message", (data) => {
            if (data instanceof Buffer) {
                // Binary
                this.HandleData(data);
            }
            else if (data instanceof ArrayBuffer) {
                // Binary arraybuffer?
                this.HandleData(Buffer.from(data));
            }
            else if (typeof data === "string") {
                this.HandleData(Buffer.from(data, "utf-8"));
            }
        });
    }
    close(code, reason) {
        this.socket.close(code, reason);
    }
    send(data) {
        this.socket.send(data);
    }
    sendTelnet(command, option, data) {
        switch (command) {
            case telnet_1.default.TelnetNegotiation.SB:
                if (data !== undefined) {
                    this.send(Buffer.from([
                        telnet_1.default.TelnetNegotiation.IAC,
                        command,
                        option,
                        ...data,
                        telnet_1.default.TelnetNegotiation.IAC,
                        telnet_1.default.TelnetNegotiation.SE,
                    ]));
                }
                else {
                    this.send(Buffer.from([
                        telnet_1.default.TelnetNegotiation.IAC,
                        command,
                        option,
                        0,
                        telnet_1.default.TelnetNegotiation.IAC,
                        telnet_1.default.TelnetNegotiation.SE,
                    ]));
                }
                break;
            default:
                this.send(Buffer.from([telnet_1.default.TelnetNegotiation.IAC, command, option]));
                break;
        }
    }
    sendSubnegotiation(option, data) {
        if (typeof data === "string") {
            data = Buffer.from(data, "utf-8");
        }
        this.sendTelnet(telnet_1.default.TelnetNegotiation.SB, option, data);
    }
    sendGMCP(namespace, data) {
        if (!this.options.HasOption(telnet_1.default.TelnetOption.GMCP))
            return;
        if (typeof data === "object") {
            // Convert object to JSON.
            data = JSON.stringify(data);
        }
        this.sendSubnegotiation(telnet_1.default.TelnetOption.GMCP, `${namespace} ${data}`);
    }
    will(option) {
        if (this.options.GetState(option) === telnet_1.default.TelnetOptionState.DISABLED) {
            this.options.SetState(option, telnet_1.default.TelnetOptionState.WAITING);
            this.sendTelnet(telnet_1.default.TelnetNegotiation.WILL, option);
        }
    }
    wont(option) {
        if (this.options.GetState(option) === telnet_1.default.TelnetOptionState.DISABLED) {
            this.options.SetState(option, telnet_1.default.TelnetOptionState.WAITING);
            this.sendTelnet(telnet_1.default.TelnetNegotiation.WONT, option);
        }
    }
    do(option) {
        if (this.options.GetState(option) === telnet_1.default.TelnetOptionState.DISABLED) {
            this.options.SetState(option, telnet_1.default.TelnetOptionState.WAITING);
            this.sendTelnet(telnet_1.default.TelnetNegotiation.DO, option);
        }
    }
    dont(option) {
        if (this.options.GetState(option) === telnet_1.default.TelnetOptionState.DISABLED) {
            this.options.SetState(option, telnet_1.default.TelnetOptionState.WAITING);
            this.sendTelnet(telnet_1.default.TelnetNegotiation.DONT, option);
        }
    }
    prompt(message, mask = false) {
        if (mask) {
            this.dont(telnet_1.default.TelnetOption.ECHO);
        }
        return new Promise((resolve, reject) => {
            this.responder = (data) => {
                if (mask) {
                    this.do(telnet_1.default.TelnetOption.ECHO);
                }
                resolve(data);
            };
            this.send(message);
        });
    }
    HandleData(data) {
        const event = telnet_1.default.ParseSequence(data);
        if (event !== null) {
            // Telnet event
            this.emit("telnet", event);
            switch (event.command) {
                case telnet_1.default.TelnetNegotiation.SB:
                    this.emit("subnegotiation", event.option, event.data || Buffer.alloc(0));
                    if (event.option === telnet_1.default.TelnetOption.GMCP && event.data !== undefined) {
                        // Process GMCP
                        const d = event.data.toString();
                        let offset = d.indexOf(" ", 0);
                        if (offset === -1)
                            offset = d.length;
                        const namespace = d.substring(0, offset);
                        const objString = offset !== d.length ? d.substring(offset + 1, d.length) : "";
                        if (objString.length > 0) {
                            try {
                                const obj = JSON.parse(objString);
                                this.emit("gmcp", namespace, obj);
                            }
                            catch {
                                this.emit("gmcp", namespace, { data: objString });
                            }
                        }
                    }
                    break;
                case telnet_1.default.TelnetNegotiation.WILL:
                    if (this.options.GetState(event.option) === telnet_1.default.TelnetOptionState.WAITING) {
                        this.options.SetState(event.option, telnet_1.default.TelnetOptionState.ENABLED);
                    }
                    this.emit("will", event.option);
                    break;
                case telnet_1.default.TelnetNegotiation.WONT:
                    if (this.options.GetState(event.option) === telnet_1.default.TelnetOptionState.WAITING) {
                        this.options.SetState(event.option, telnet_1.default.TelnetOptionState.DISABLED);
                    }
                    this.emit("wont", event.option);
                    break;
                case telnet_1.default.TelnetNegotiation.DO:
                    if (this.options.GetState(event.option) === telnet_1.default.TelnetOptionState.WAITING) {
                        this.options.SetState(event.option, telnet_1.default.TelnetOptionState.ENABLED);
                    }
                    this.emit("do", event.option);
                    break;
                case telnet_1.default.TelnetNegotiation.DONT:
                    if (this.options.GetState(event.option) === telnet_1.default.TelnetOptionState.WAITING) {
                        this.options.SetState(event.option, telnet_1.default.TelnetOptionState.DISABLED);
                    }
                    this.emit("dont", event.option);
                    break;
            }
        }
        else {
            const message = data.toString("utf-8");
            if (this.responder !== null) {
                this.responder(message);
                this.responder = null;
            }
            else {
                this.emit("message", message);
            }
        }
    }
    on(event, listener) {
        return super.on(event, listener);
    }
    emit(event, ...args) {
        return super.emit(event, ...args);
    }
}
(function (WST) {
    class Server extends events_1.EventEmitter {
        constructor(host, port, secure) {
            super();
            this.clients = new Map();
            let server;
            if (secure !== undefined) {
                server = https_1.default.createServer(secure);
            }
            else {
                server = http_1.default.createServer();
            }
            this._server = new ws_1.default.Server({
                server,
            });
            this._server.on("listening", () => this.emit("listening", host, port));
            this._server.on("connection", (socket) => {
                let client = new WST(socket);
                const uuid = client.UUID;
                this.clients.set(client.UUID, client);
                client.on("close", (code, reason) => {
                    this.clients.delete(uuid);
                    client = null;
                });
                this.emit("connection", client);
            });
            this._server.on("error", (err) => this.emit("error", err));
            server.listen(port, host);
        }
        on(event, listener) {
            return super.on(event, listener);
        }
        emit(event, ...args) {
            return super.emit(event, ...args);
        }
    }
    WST.Server = Server;
    WST.Telnet = telnet_1.default;
})(WST || (WST = {}));
module.exports = WST;
